@@ -37,9 +37,10 @@ using std::endl;
 #include "psbrt.h"
 #include "DPMLIOmutau.h"
 #include "DPMLIOneal8.h"
-#include "cnft.h"
-#include "cprnft.h"
-#include "cpsambrt_predict.h"
+//#include "cnft.h"
+//#include "cprnft.h"
+//#include "cpsambrt_predict.h"
 #include "cnft2.h"
+#include "getpred.h"
 #include "cprnft2.h"
 #include "cpsambrt_predict2.h"
